"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.execute = void 0;
const vscode = require("vscode-languageserver");
const html2pug_1 = require("@volar/html2pug");
function execute(document, sourceFile, connection) {
    const desc = sourceFile.getDescriptor();
    if (!desc.template)
        return;
    const lang = desc.template.lang;
    if (lang !== 'pug')
        return;
    const html = html2pug_1.pugToHtml(desc.template.content);
    const newTemplate = `<template>\n` + html + `\n`;
    let start = desc.template.loc.start - '<template>'.length;
    const end = desc.template.loc.end;
    const startMatch = '<template';
    while (!document.getText(vscode.Range.create(document.positionAt(start), document.positionAt(start + startMatch.length))).startsWith(startMatch)) {
        start--;
        if (start < 0) {
            throw `Can't find start of tag <template>`;
        }
    }
    const range = vscode.Range.create(document.positionAt(start), document.positionAt(end));
    const textEdit = vscode.TextEdit.replace(range, newTemplate);
    connection.workspace.applyEdit({ changes: { [document.uri]: [textEdit] } });
}
exports.execute = execute;
//# sourceMappingURL=pugToHtml.js.map