"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.isRefType = exports.isBlacklistNode = exports.register = void 0;
const vscode = require("vscode-languageserver");
const shared = require("@volar/shared");
function register({ modules: { typescript: ts }, sourceFiles, getTsLs }) {
    const asts = new WeakMap();
    return (document, position) => {
        for (const tsLoc of sourceFiles.toTsLocations(document.uri, position)) {
            if (tsLoc.lsType === 'template')
                continue;
            if (tsLoc.type === 'embedded-ts' && !tsLoc.range.data.capabilities.completion)
                continue;
            const tsLs = getTsLs(tsLoc.lsType);
            const tsDoc = tsLs.__internal__.getTextDocument(tsLoc.uri);
            if (!tsDoc)
                continue;
            const sourceFile = getAst(tsDoc);
            if (isBlacklistNode(ts, sourceFile, tsDoc.offsetAt(tsLoc.range.start)))
                continue;
            const typeDefs = tsLs.findTypeDefinition(tsLoc.uri, tsLoc.range.start);
            if (isRefType(typeDefs, tsLs)) {
                return '${1:.value}';
            }
        }
    };
    function getAst(tsDoc) {
        let ast = asts.get(tsDoc);
        if (!ast) {
            ast = ts.createSourceFile(shared.uriToFsPath(tsDoc.uri), tsDoc.getText(), ts.ScriptTarget.Latest);
            asts.set(tsDoc, ast);
        }
        return ast;
    }
}
exports.register = register;
function isBlacklistNode(ts, node, pos) {
    if (ts.isVariableDeclaration(node) && pos >= node.name.getFullStart() && pos <= node.name.getEnd()) {
        return true;
    }
    else if (ts.isFunctionDeclaration(node) && node.name && pos >= node.name.getFullStart() && pos <= node.name.getEnd()) {
        return true;
    }
    else if (ts.isParameter(node) && pos >= node.name.getFullStart() && pos <= node.name.getEnd()) {
        return true;
    }
    else if (ts.isPropertyAssignment(node) && pos >= node.name.getFullStart() && pos <= node.name.getEnd()) {
        return true;
    }
    else if (ts.isShorthandPropertyAssignment(node)) {
        return true;
    }
    else if (ts.isImportDeclaration(node)) {
        return true;
    }
    else if (ts.isLiteralTypeNode(node)) {
        return true;
    }
    else {
        let _isBlacklistNode = false;
        node.forEachChild(node => {
            if (_isBlacklistNode)
                return;
            if (pos >= node.getFullStart() && pos <= node.getEnd()) {
                if (isBlacklistNode(ts, node, pos)) {
                    _isBlacklistNode = true;
                }
            }
        });
        return _isBlacklistNode;
    }
}
exports.isBlacklistNode = isBlacklistNode;
function isRefType(typeDefs, tsLs) {
    for (const typeDefine of typeDefs) {
        const uri = vscode.Location.is(typeDefine) ? typeDefine.uri : typeDefine.targetUri;
        const range = vscode.Location.is(typeDefine) ? typeDefine.range : typeDefine.targetSelectionRange;
        if (uri.endsWith('reactivity.d.ts')) {
            const defineDoc = tsLs.__internal__.getTextDocument(uri);
            if (!defineDoc)
                continue;
            const typeName = defineDoc.getText(range);
            switch (typeName) {
                case 'Ref':
                case 'ComputedRef':
                case 'WritableComputedRef':
                    return true;
            }
        }
    }
    return false;
}
exports.isRefType = isRefType;
//# sourceMappingURL=refAutoClose.js.map