"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useSfcEntryForTemplateLs = void 0;
const vscode_languageserver_textdocument_1 = require("vscode-languageserver-textdocument");
const reactivity_1 = require("@vue/reactivity");
const SourceMaps = require("../utils/sourceMaps");
const upath = require("upath");
const string_1 = require("../utils/string");
const shared = require("@volar/shared");
function useSfcEntryForTemplateLs(getUnreactiveDoc, script, scriptSetup, template) {
    let version = 0;
    const textDocument = reactivity_1.computed(() => {
        const vueDoc = getUnreactiveDoc();
        const uri = `${vueDoc.uri}.ts`;
        const vueFileName = upath.basename(shared.uriToFsPath(vueDoc.uri));
        let content = '';
        if (scriptSetup.value || script.value) {
            content += `import { __VLS_options, __VLS_name } from './${vueFileName}.__VLS_script';\n`;
            content += `export { __VLS_options, __VLS_name } from './${vueFileName}.__VLS_script';\n`;
            content += `export * from './${vueFileName}.__VLS_script';\n`;
            if (scriptSetup.value) {
                content += `import { __VLS_component } from './${vueFileName}.__VLS_script';\n`;
                content += `export { __VLS_component } from './${vueFileName}.__VLS_script';\n`;
            }
            else if (script.value) {
                content += `import __VLS_component_1 from './${vueFileName}.__VLS_script';\n`;
                content += `import { __VLS_component as __VLS_component_2 } from './${vueFileName}.__VLS_script';\n`;
                content += `export declare var __VLS_component: typeof __VLS_component_1 extends (new (...args: infer _1) => infer _2)\n`;
                content += `    ? typeof __VLS_component_1 : typeof __VLS_component_1 extends ((...args: infer _3) => infer _4)\n`;
                content += `    ? typeof __VLS_component_1 : typeof __VLS_component_2;\n`;
            }
        }
        else {
            content += `export var __VLS_options = {};\n`;
            content += `export var __VLS_name = undefined;\n`;
            content += `export var __VLS_component = __VLS_defineComponent({});\n`;
        }
        content += `declare var __VLS_ctx: __VLS_PickNotAny<InstanceType<typeof __VLS_component>, ReturnType<typeof __VLS_component>>;\n`;
        content += `declare var __VLS_ComponentsWrap: typeof __VLS_options & { components: { } };\n`;
        content += `declare var __VLS_Components: typeof __VLS_ComponentsWrap.components & __VLS_GlobalComponents & __VLS_PickComponents<typeof __VLS_ctx> & __VLS_SelfComponent<typeof __VLS_name, typeof __VLS_component>;\n`;
        content += `__VLS_ctx.${string_1.SearchTexts.Context};\n`;
        content += `__VLS_Components.${string_1.SearchTexts.Components};\n`;
        content += `__VLS_options.setup().${string_1.SearchTexts.SetupReturns};\n`;
        content += `__VLS_options.props.${string_1.SearchTexts.Props};\n`;
        content += `\n`;
        content += `export default {} as typeof __VLS_component & {\n`;
        content += `__VLS_raw: typeof __VLS_component\n`;
        content += `__VLS_options: typeof __VLS_options,\n`;
        content += template.value ? `__VLS_slots: typeof import ('./${vueFileName}.__VLS_template').default,\n` : `// no template\n`;
        content += `};\n`;
        return vscode_languageserver_textdocument_1.TextDocument.create(uri, 'typescript', version++, content);
    });
    const sourceMap = reactivity_1.computed(() => {
        if (textDocument.value) {
            const vueDoc = getUnreactiveDoc();
            const docText = textDocument.value.getText();
            const sourceMap = new SourceMaps.TsSourceMap(vueDoc, textDocument.value, 'template', false, {
                foldingRanges: false,
                formatting: false,
                documentSymbol: false,
                codeActions: false,
            });
            sourceMap.add({
                data: {
                    vueTag: 'sfc',
                    capabilities: {},
                },
                mode: SourceMaps.Mode.Overlap,
                sourceRange: {
                    start: 0,
                    end: 0,
                },
                mappedRange: {
                    start: 0,
                    end: docText.length,
                },
            });
            return sourceMap;
        }
    });
    return {
        textDocument,
        sourceMap,
    };
}
exports.useSfcEntryForTemplateLs = useSfcEntryForTemplateLs;
//# sourceMappingURL=useSfcEntryForTemplateLs.js.map